@@ -4,14 +4,17 @@ AC_VERSION=
 AUTOMAKE=${AUTOMAKE:-automake}
 AM_INSTALLED_VERSION=$($AUTOMAKE --version | sed -e '2,$ d' -e 's/.* \([0-9]*\.[0-9]*\).*/\1/')
 
-if [ "$AM_INSTALLED_VERSION" != "1.10" \
-    -a "$AM_INSTALLED_VERSION" != "1.11" ];then
+case "$AM_INSTALLED_VERSION" in
+    1.1[0-9])
+	;;
+    *)
 	echo
-	echo "You must have automake > 1.10 or 1.11 installed to compile lxde-settings-daemon."
+	echo "You must have automake >= 1.10 installed to compile lxmusic."
 	echo "Install the appropriate package for your distribution,"
 	echo "or get the source tarball at http://ftp.gnu.org/gnu/automake/"
 	exit 1
-fi
+	;;
+esac
 
 set -x
 
@@ -21,7 +24,6 @@ fi
 
 ${ACLOCAL:-aclocal$AM_VERSION} ${ACLOCAL_ARG}
 ${AUTOHEADER:-autoheader$AC_VERSION} --force
-AUTOMAKE=$AUTOMAKE libtoolize -c --automake --force
 AUTOMAKE=$AUTOMAKE intltoolize -c --automake --force
 $AUTOMAKE --add-missing --copy --include-deps --foreign
 ${AUTOCONF:-autoconf$AC_VERSION}
