@@ -38,9 +38,7 @@
 #include <gdk/gdkx.h>
 #include <gdk/gdkkeysyms.h>
 
-#ifdef HAVE_LIBNOTIFY
 #include "lxmusic-notify.h"
-#endif
 
 #include "lxmusic-plugin-config.h"
 #include "utils.h"
@@ -75,20 +73,24 @@ typedef struct _TrackProperties{
     const char *url;
     const char *mime;
     const char *comment;
+    const char *picture_front;
+    
     int32_t duration;
     int32_t isvbr;
     int32_t bitrate;
     int32_t size;
 }TrackProperties;
 
-static void 	send_notifcation			( const gchar *artist, const gchar* title );
-static int 	update_track				( xmmsv_t *value, GtkTreeIter* it );
+static void 	send_notification			( LXMusicNotification lxn );
+static void 	send_notification_pixbuf		( LXMusicNotification lxn, GdkPixbuf *pixbuf );
+static int 	update_track				( xmmsv_t *value, GtkTreeRowReference* ref );
 static int 	on_coll_info_received			( xmmsv_t* value, void* user_data );
-
-
-#ifdef HAVE_LIBNOTIFY
-static LXMusic_Notification *lxmusic_notification = NULL;
-#endif 
+static int 	on_picture_front_received		( xmmsv_t* value, void* user_data );
+static int 	on_playback_started			( xmmsv_t* value, void* user_data );
+void		on_prev_btn_clicked			( GtkButton* btn, gpointer user_data );
+void		on_next_btn_clicked			( GtkButton* btn, gpointer user_data );
+void		on_play_btn_clicked			( GtkButton* btn, gpointer user_data );
+void		on_stop_btn_clicked			( GtkButton* btn, gpointer user_data );
 
 static xmmsc_connection_t *con = NULL;
 static GtkWidget *main_win = NULL;
@@ -143,6 +145,9 @@ static int win_ypos = 0;
 void 			on_locate_cur_track	(GtkAction* act, gpointer user_data);
 void 			on_play_btn_clicked	(GtkButton* btn, gpointer user_data);
 
+static gboolean 	on_tray_icon_middle_clicked	(GtkWidget *widget, GdkEventButton *event, gpointer data);
+static void 		on_volume_btn_scrolled		(GtkWidget *widget, GdkEventScroll *event, gpointer user_data);
+
 static GtkTreeIter	get_current_track_iter	();
 static gboolean		get_track_properties 	(xmmsv_t *value, TrackProperties *properties);
 
@@ -201,7 +206,9 @@ static void save_config()
 
 static int on_xmms_quit(xmmsv_t *value, void *user_data)
 {
-    gtk_widget_destroy(main_win);
+    if (main_win)
+        gtk_widget_destroy(main_win);
+    main_win = NULL;
     gtk_main_quit();
     return TRUE;
 }
@@ -214,10 +221,6 @@ void on_quit(GtkAction* act, gpointer user_data)
     if(tray_icon)
         g_object_unref(tray_icon);
 
-#ifdef HAVE_LIBNOTIFY
-    lxmusic_notification_free( lxmusic_notification );
-#endif 
-
     if( ! play_after_exit )
     {
         /* quit the server */
@@ -227,6 +230,7 @@ void on_quit(GtkAction* act, gpointer user_data)
     else
     {
         gtk_widget_destroy(main_win);
+        main_win = NULL;
         gtk_main_quit();
     }
 }
@@ -252,32 +256,38 @@ void on_main_win_destroy(GtkWidget* win)
     }
 }
 
-static void open_url(GtkAboutDialog* dlg, const char* url, gpointer user_data)
-{
-    const char* argv[] = {"xdg-open", NULL, NULL};
-    argv[1] = url;
-    g_spawn_async("/", (gchar**)argv, NULL, G_SPAWN_SEARCH_PATH, NULL, NULL, NULL, NULL);
-}
-
 void on_about(GtkWidget* mi, gpointer data)
 {
     const char* authors[] = { "洪任諭 (Hong Jen Yee) <pcman.tw@gmail.com>", 
 			      "Jürgen Hötzel <juergen@archlinux.org>", NULL };
-    const char* artists[] = { N_("Official icon of xmms2 by Arnaud DIDRY"), NULL };
+    const char* artists[] = { _("Official icon of xmms2 by Arnaud DIDRY"), NULL };
     GtkWidget* about;
 
-    gtk_about_dialog_set_url_hook(open_url, NULL, NULL);
-
     about = gtk_about_dialog_new();
-    gtk_about_dialog_set_name( (GtkAboutDialog*)about, "LXMusic" );
+    gtk_about_dialog_set_program_name( (GtkAboutDialog*)about, "LXMusic" );
     gtk_about_dialog_set_logo_icon_name((GtkAboutDialog*)about, "lxmusic");
     gtk_about_dialog_set_version( (GtkAboutDialog*)about, VERSION );
     gtk_about_dialog_set_authors( (GtkAboutDialog*)about, authors );
     gtk_about_dialog_set_artists( (GtkAboutDialog*)about, artists );
-    gtk_about_dialog_set_comments( (GtkAboutDialog*)about, _("Music Player for LXDE\nSimple GUI XMMS2 client") );
-    gtk_about_dialog_set_license( (GtkAboutDialog*)about, "GNU General Public License" );
-    gtk_about_dialog_set_website( (GtkAboutDialog*)about, "http://lxde.org/" );
+    gtk_about_dialog_set_comments( (GtkAboutDialog*)about, _("Music Player for LXDE\nSimple XMMS2 GUI client") );
+    gtk_about_dialog_set_license( (GtkAboutDialog*)about,
+    "Copyright \xC2\xA9 2006 Hong Jen Yee (PCMan)\n\n"
+    "This program is free software; you can redistribute it and/or modify\n"
+    "it under the terms of the GNU General Public License as published by\n"
+    "the Free Software Foundation; either version 2 of the License, or\n"
+    "(at your option) any later version.\n"
+    "\n"
+    "This program is distributed in the hope that it will be useful,\n"
+    "but WITHOUT ANY WARRANTY; without even the implied warranty of\n"
+    "MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.\n"
+    "See the GNU General Public License for more details.\n"
+    "\n"
+    "You should have received a copy of the GNU General Public License\n"
+    "along with this program; if not, write to the Free Software Foundation,\n"
+    "Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA" );
+    gtk_about_dialog_set_website( (GtkAboutDialog*)about, "https://www.lxde.org/" );
     gtk_window_set_transient_for( (GtkWindow*)about, (GtkWindow*)main_win );
+    gtk_window_set_destroy_with_parent((GtkWindow*)about, TRUE);
     gtk_dialog_run( (GtkDialog*)about );
     gtk_widget_destroy( about );
 }
@@ -348,7 +358,7 @@ static int on_pref_dlg_init_widget(xmmsv_t* value, void* user_data)
             gtk_spin_button_set_value(GTK_SPIN_BUTTON(w), atoi(val));
         else if( GTK_IS_ENTRY(w) )
             gtk_entry_set_text(GTK_ENTRY(w), val);
-        else if( GTK_IS_COMBO_BOX_ENTRY(w) )
+        else if( GTK_IS_COMBO_BOX(w) && gtk_bin_get_child(GTK_BIN(w)) != NULL )
             gtk_entry_set_text(GTK_ENTRY(gtk_bin_get_child(GTK_BIN(w))), val);
         else
             g_debug("%s is not supported", G_OBJECT_TYPE_NAME(w));
@@ -381,7 +391,15 @@ int on_pref_dlg_init_output_plugin(xmmsv_t* value, void* user_data)
 	/* fallback to xmms2 plugin names if no translation available*/
 	if ( label == NULL )
 	    label = xmms2_name;
+#if GTK_CHECK_VERSION(2, 24, 0)
+	GtkListStore *store = GTK_LIST_STORE(gtk_combo_box_get_model(output_plugin_cb));
+	GtkTreeIter iter;
+	/* keeping it as GtkComboBox for backward compatibility with GTK+ < 2.24 */
+	gtk_list_store_append(store, &iter);
+	gtk_list_store_set(store, &iter, 0, label, -1);
+#else
 	gtk_combo_box_append_text( output_plugin_cb, label );
+#endif
     }
     
 
@@ -398,7 +416,7 @@ int on_pref_dlg_init_output_plugin(xmmsv_t* value, void* user_data)
 static void on_tray_icon_activate(GtkStatusIcon* icon, gpointer user_data)
 {
     /* FIXME: should we unload the playlist to free resources here? */
-    if( GTK_WIDGET_VISIBLE(main_win) )
+    if( gtk_widget_get_visible(main_win) )
     {
 		/* save window position before we hide the window */
 		gtk_window_get_position((GtkWindow*)main_win, &win_xpos, &win_ypos);
@@ -421,25 +439,67 @@ void on_show_main_win(GtkAction* act, gpointer user_data)
 static void on_tray_icon_popup_menu(GtkStatusIcon* icon, guint btn, guint time, gpointer user_data)
 {
 	/* init tray icon widgets */
-	GtkBuilder *builder = gtk_builder_new ();
-	if(gtk_builder_add_from_file(builder, PACKAGE_DATA_DIR "/lxmusic/popup.ui.glade", NULL))
-	{
-        	GtkWidget *tray_play_btn = (GtkWidget*)gtk_builder_get_object(builder, "play");
-		GtkWidget *tray_popup = (GtkWidget*)gtk_builder_get_object(builder, "popup");
-        	gtk_builder_connect_signals(builder, NULL);
-		switch (playback_status)
+	GtkMenu *systray_menu;
+	GtkWidget *item = NULL;
+
+	systray_menu = GTK_MENU (gtk_menu_new());
+
+	item = gtk_image_menu_item_new_from_stock(GTK_STOCK_OPEN, NULL);
+	gtk_menu_item_set_label (GTK_MENU_ITEM(item), _("Show LXMusic"));
+	gtk_menu_shell_append(GTK_MENU_SHELL(systray_menu), item);
+	g_signal_connect (G_OBJECT (item), "activate",
+			  G_CALLBACK(on_show_main_win),
+			  NULL);
+
+	item = gtk_separator_menu_item_new();
+	gtk_menu_shell_append(GTK_MENU_SHELL(systray_menu), item);
+
+	switch (playback_status)
 		{
 			case XMMS_PLAYBACK_STATUS_PLAY:
-				g_object_set ( (GObject*)tray_play_btn, "stock-id", "gtk-media-pause", NULL);
+				item = gtk_image_menu_item_new_from_stock(GTK_STOCK_MEDIA_PAUSE, NULL);
 				break;
 			case XMMS_PLAYBACK_STATUS_PAUSE:
 			case XMMS_PLAYBACK_STATUS_STOP:
-				g_object_set ( (GObject*)tray_play_btn, "stock-id", "gtk-media-play", NULL);
+				item = gtk_image_menu_item_new_from_stock(GTK_STOCK_MEDIA_PLAY, NULL);
 				break;
 		}
-		gtk_menu_popup((GtkMenu*)tray_popup, NULL, NULL, NULL, NULL, btn, time);
-	}
-	g_object_unref(builder);
+
+	gtk_menu_shell_append(GTK_MENU_SHELL(systray_menu), item);
+	g_signal_connect (G_OBJECT (item), "activate",
+			  G_CALLBACK(on_play_btn_clicked),
+			  NULL);
+
+	item = gtk_image_menu_item_new_from_stock(GTK_STOCK_MEDIA_STOP, NULL);
+	gtk_menu_shell_append(GTK_MENU_SHELL(systray_menu), item);
+	g_signal_connect (G_OBJECT (item), "activate",
+			  G_CALLBACK(on_stop_btn_clicked),
+			  NULL);
+
+	item = gtk_image_menu_item_new_from_stock(GTK_STOCK_MEDIA_PREVIOUS, NULL);
+	gtk_menu_shell_append(GTK_MENU_SHELL(systray_menu), item);
+	g_signal_connect (G_OBJECT (item), "activate",
+			  G_CALLBACK(on_prev_btn_clicked),
+			  NULL);
+
+	item = gtk_image_menu_item_new_from_stock(GTK_STOCK_MEDIA_NEXT, NULL);
+	gtk_menu_shell_append(GTK_MENU_SHELL(systray_menu), item);
+	g_signal_connect (G_OBJECT (item), "activate",
+			  G_CALLBACK(on_next_btn_clicked),
+			  NULL);
+
+	item = gtk_separator_menu_item_new();
+	gtk_menu_shell_append(GTK_MENU_SHELL(systray_menu), item);
+
+	item = gtk_image_menu_item_new_from_stock(GTK_STOCK_QUIT, NULL);
+	gtk_menu_shell_append(GTK_MENU_SHELL(systray_menu), item);
+	g_signal_connect (G_OBJECT (item), "activate",
+			  G_CALLBACK(on_quit),
+			  NULL);
+
+	/* show */
+	gtk_widget_show_all(GTK_WIDGET(systray_menu));
+	gtk_menu_popup(GTK_MENU(systray_menu), NULL, NULL, NULL, NULL, btn, time);
 
 	return;
 }
@@ -447,9 +507,31 @@ static void on_tray_icon_popup_menu(GtkStatusIcon* icon, guint btn, guint time,
 static void create_tray_icon()
 {
     tray_icon = (GtkWidget*)gtk_status_icon_new_from_icon_name("lxmusic");
-    gtk_status_icon_set_tooltip(GTK_STATUS_ICON(tray_icon), _("LXMusic"));
+    gtk_status_icon_set_tooltip_text(GTK_STATUS_ICON(tray_icon), _("LXMusic"));
     g_signal_connect(tray_icon, "activate", G_CALLBACK(on_tray_icon_activate), NULL );
     g_signal_connect(tray_icon, "popup-menu", G_CALLBACK(on_tray_icon_popup_menu), NULL );
+    g_signal_connect(tray_icon, "scroll-event", G_CALLBACK(on_volume_btn_scrolled), volume_btn);
+    g_signal_connect(tray_icon, "button_press_event", G_CALLBACK(on_tray_icon_middle_clicked), NULL );
+ }
+
+static gboolean on_tray_icon_middle_clicked(GtkWidget *widget, GdkEventButton *event, gpointer data) { 
+    /* Only handle the keypress event on middle click Toggle play/pause */
+    if ( event->button == 2 ) {
+	xmmsc_result_t *res;
+	if( playback_status == XMMS_PLAYBACK_STATUS_PLAY )
+	{
+	    res = xmmsc_playback_pause(con);
+	    xmmsc_result_notifier_set_and_unref(res, on_playback_started, NULL);
+	}
+	else
+	{
+	    res = xmmsc_playback_start(con);
+	    xmmsc_result_unref(res);
+	}
+	return TRUE;
+	
+    } 
+    return FALSE;
 }
 
 void on_preference(GtkAction* act, gpointer data)
@@ -490,6 +572,8 @@ void on_preference(GtkAction* act, gpointer data)
         xmmsc_result_notifier_set_full(res, on_pref_dlg_init_widget, g_object_ref(id3v1_encoding), g_object_unref );
         xmmsc_result_unref(res);
 
+        gtk_window_set_transient_for(GTK_WINDOW(dlg), (GtkWindow*)main_win);
+        gtk_window_set_destroy_with_parent(GTK_WINDOW(dlg), TRUE);
         if( gtk_dialog_run(GTK_DIALOG(dlg)) == GTK_RESPONSE_OK )
         {
             int i;
@@ -576,9 +660,7 @@ static int on_playlist_coll_received(xmmsv_t* value, void* user_data)
 
 static int on_coll_info_received(xmmsv_t* value, void* user_data) 
 {
-    xmmsv_list_iter_t *l_iter;
     int32_t id;
-    const gchar *str;
     GtkTreeModel* model =  GTK_TREE_MODEL(list_store);
     GtkTreeIter it;
     int i = 0;
@@ -591,9 +673,11 @@ static int on_coll_info_received(xmmsv_t* value, void* user_data)
 	xmmsv_t *id_val;
 	int32_t id;
 	xmmsv_list_get (value, i,  &track_info);
-	xmmsv_dict_get ( track_info, "id", &id_val );
-	xmmsv_get_int ( id_val, &id );
-	g_hash_table_insert( id_to_coll_info, GINT_TO_POINTER( id ), track_info );
+	if (xmmsv_dict_get ( track_info, "id", &id_val ))
+	{
+	    xmmsv_get_int ( id_val, &id );
+	    g_hash_table_insert( id_to_coll_info, GINT_TO_POINTER( id ), track_info );
+	}
     }
 
     gtk_tree_model_get_iter_first( model, &it );
@@ -601,15 +685,20 @@ static int on_coll_info_received(xmmsv_t* value, void* user_data)
     while (gtk_list_store_iter_is_valid( list_store , &it )) 
     {
 	xmmsv_t *track_info;
-	TrackProperties track_properties;
+        GtkTreePath *path = gtk_tree_model_get_path(GTK_TREE_MODEL(list_store), &it);
+        GtkTreeRowReference *ref = gtk_tree_row_reference_new(GTK_TREE_MODEL(list_store), path);
+        gtk_tree_path_free(path);
 	gtk_tree_model_get( model, &it, COL_ID, &id, -1 );
 	/* find corressponding track info */
 	track_info = (xmmsv_t*) g_hash_table_lookup( id_to_coll_info, GINT_TO_POINTER( id ));
-	update_track( track_info, &it ); 
+	update_track( track_info, ref );
+	gtk_tree_row_reference_free(ref);
 	gtk_tree_model_iter_next( model, &it );
 	i++;
     }
     g_hash_table_unref( id_to_coll_info );
+
+    return 0;
 }
 
 
@@ -666,7 +755,11 @@ static int on_track_info_received(xmmsv_t* value, void* user_data)
     if (tp.size){
 	char buf[100];
 	char *size_str;
+#if GLIB_CHECK_VERSION(2, 30, 0)
+        size_str = g_format_size(tp.size);
+#else
 	size_str = g_format_size_for_display( tp.size );
+#endif
 	strcpy( buf, size_str );
 	g_free( size_str );
 
@@ -809,7 +902,11 @@ gboolean on_playlist_view_drag_drop(GtkWidget      *widget,
     /*  Don't call the default handler  */
 //    g_signal_stop_emission_by_name( widget, "drag-drop" );
 
+#if GTK_CHECK_VERSION(2, 22, 0)
+    if( g_list_find( gdk_drag_context_list_targets(drag_ctx), target ) )
+#else
     if( g_list_find( drag_ctx->targets, target ) )
+#endif
     {
         gtk_drag_get_data( widget, drag_ctx, target, time );
         return TRUE;
@@ -971,6 +1068,7 @@ void on_add_files( GtkMenuItem* item, gpointer user_data )
                                                   GTK_STOCK_ADD, RESPONSE_ADD, NULL );
     GtkFileFilter* filter;
 
+    gtk_window_set_destroy_with_parent(GTK_WINDOW(dlg), TRUE);
     gtk_file_chooser_set_select_multiple( (GtkFileChooser*)dlg, TRUE );
 
     /* add a custom filter which filters autio files */
@@ -1011,7 +1109,8 @@ void on_add_url( GtkMenuItem* item, gpointer user_data )
             GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
             GTK_STOCK_OK, GTK_RESPONSE_OK, NULL );
     GtkWidget *entry = gtk_entry_new();
-    gtk_box_pack_start( (GtkBox*)((GtkDialog*)dlg)->vbox, entry, FALSE, FALSE, 4 );
+    gtk_window_set_destroy_with_parent(GTK_WINDOW(dlg), TRUE);
+    gtk_box_pack_start( (GtkBox*)gtk_dialog_get_content_area(GTK_DIALOG(dlg)), entry, FALSE, FALSE, 4 );
     gtk_dialog_set_default_response( (GtkDialog*)dlg, GTK_RESPONSE_OK );
     gtk_entry_set_activates_default( (GtkEntry*)entry, TRUE );
     gtk_widget_show_all( dlg );
@@ -1037,7 +1136,7 @@ void on_add_btn_clicked(GtkButton* btn, gpointer user_data)
 
 static int intcmp( gconstpointer a, gconstpointer b )
 {
-    return (int)b - (int)a;
+    return GPOINTER_TO_INT(b) - GPOINTER_TO_INT(a);
 }
 
 void on_remove_all(GtkAction* act, gpointer user_data)
@@ -1061,7 +1160,7 @@ void on_remove_selected(GtkAction* act, gpointer user_data)
         for( sel = sels; sel; sel = sel->next )
         {
             GtkTreePath* path = (GtkTreePath*)sel->data;
-            sel->data = (gpointer)gtk_tree_path_get_indices( path )[0];
+            sel->data = GINT_TO_POINTER(gtk_tree_path_get_indices( path )[0]);
             gtk_tree_path_free( path );
         }
 
@@ -1074,7 +1173,7 @@ void on_remove_selected(GtkAction* act, gpointer user_data)
         for( sel = sels; sel; sel = sel->next )
         {
             xmmsc_result_t* res;
-            int pos = (int)sel->data;
+            int pos = GPOINTER_TO_INT(sel->data);
             res = xmmsc_playlist_remove_entry( con, cur_playlist, pos );
             xmmsc_result_unref( res );
         }
@@ -1092,11 +1191,19 @@ gboolean  on_playlist_view_key_press_event (GtkWidget *widget,
 {
     switch ( event->keyval) 
     {
+#if GTK_CHECK_VERSION(2, 21, 0)
+    case GDK_KEY_Delete:
+#else
     case GDK_Delete:
+#endif
 	/* dummy values: needs cleanup  */
 	on_remove_selected (NULL, NULL); 
 	break;
+#if GTK_CHECK_VERSION(2, 21, 0)
+    case GDK_KEY_Insert:
+#else
     case GDK_Insert:
+#endif
 	/* dummy values: needs cleanup  */
 	on_add_files( NULL, NULL );
 	break;
@@ -1196,38 +1303,58 @@ static void render_num( GtkTreeViewColumn* col, GtkCellRenderer* render,
     g_object_set( render, "text", buf, NULL );
 }
 
-static int update_track( xmmsv_t *value, GtkTreeIter* it )
+static int update_track( xmmsv_t *value, GtkTreeRowReference *ref )
 {
     TrackProperties track_properties;
-    gboolean current_track_updated;
     char time_buf[32];
     int32_t id;
     gchar *guessed_title = NULL;
+    GtkTreePath *path;
+    GtkTreeIter it;
+    gboolean ok;
+
     if( xmmsv_is_error ( value ) ) {
         return FALSE;
     }
 
+    if (!gtk_tree_row_reference_valid(ref)) /* track was removed from the list */
+        return FALSE;
+    path = gtk_tree_row_reference_get_path(ref);
+    ok = gtk_tree_model_get_iter(GTK_TREE_MODEL(list_store), &it, path);
+    gtk_tree_path_free(path);
+    if (!ok)
+        return FALSE;
     if (!get_track_properties( value, &track_properties)) 
 	track_properties.title = guessed_title = guess_title_from_url( track_properties.url );
     timeval_to_str( track_properties.duration/1000, time_buf, G_N_ELEMENTS(time_buf) );
 
-    gtk_list_store_set( list_store, it,
+    gtk_list_store_set( list_store, &it,
                         COL_ARTIST, track_properties.artist,
                         COL_ALBUM, track_properties.album,
                         COL_TITLE, track_properties.title,
                         COL_LEN, time_buf, -1 );
 
-    
-    gtk_tree_model_get( GTK_TREE_MODEL(list_store), it, COL_ID, &id, -1 );
-     current_track_updated = id == cur_track_id;
-     if ( current_track_updated ) 
+
+    gtk_tree_model_get( GTK_TREE_MODEL(list_store), &it, COL_ID, &id, -1 );
+    if (id > 0)
     {
-	/* send desktop notification if current track was updated */  
-	send_notifcation( track_properties.artist, track_properties.title );
+	LXMusicNotification lxn = lxmusic_do_notify_prepare ( track_properties.artist, track_properties.title,  
+							      _("Now Playing:"), GTK_STATUS_ICON(tray_icon) );
+	if (!track_properties.picture_front) 
+	    /* send desktop notification if current track was updated */  
+	    send_notification( lxn );
+	else 
+	{
+	    /* postpone notification: wait for album art */
+	    xmmsc_result_t* res = xmmsc_bindata_retrieve( con, track_properties.picture_front );
+	    /* we need to keep a reference on track property strings because we pass it down here */
+	    xmmsc_result_notifier_set_full (res, on_picture_front_received, lxn, NULL);
+	    xmmsc_result_unref( res );
+	}
 	if( tray_icon ) 
 	{
 	    GString* tray_tooltip = create_window_title(track_properties.artist, track_properties.title, playback_status == XMMS_PLAYBACK_STATUS_PLAY);
-	    gtk_status_icon_set_tooltip( GTK_STATUS_ICON(tray_icon), tray_tooltip->str );
+	    gtk_status_icon_set_tooltip_text( GTK_STATUS_ICON(tray_icon), tray_tooltip->str );
 	    g_string_free( tray_tooltip, TRUE );
 	}
 	
@@ -1236,6 +1363,27 @@ static int update_track( xmmsv_t *value, GtkTreeIter* it )
     return FALSE;
 }
 
+static int on_picture_front_received ( xmmsv_t* value, void* user_data ) 
+{
+    const unsigned char *bin_data;
+    LXMusicNotification lxn = (LXMusicNotification) user_data;
+    unsigned int bin_len;
+    g_assert(xmmsv_get_bin( value, &bin_data, &bin_len ) );
+    GInputStream *picture_front_stream = g_memory_input_stream_new_from_data (bin_data, bin_len, NULL );
+    GdkPixbuf *picture_front_pixbuf =  gdk_pixbuf_new_from_stream( G_INPUT_STREAM(picture_front_stream), NULL, NULL );
+    g_object_unref (picture_front_stream);
+    if (picture_front_pixbuf) 
+    {
+	send_notification_pixbuf( lxn, picture_front_pixbuf );
+	g_object_unref( picture_front_pixbuf );
+    }
+    return TRUE;
+}
+
+
+/* helper macro to access struct member based on string key */
+#define MAYBE_GET_PROPERTY(properties, member, key_str, val) if (strcmp( key_str, #member ) == 0) val = &(properties->member); 
+
 static gboolean get_track_properties (xmmsv_t *value, TrackProperties *properties)  
 {
     /* traverse the dict of dict */
@@ -1260,29 +1408,18 @@ static gboolean get_track_properties (xmmsv_t *value, TrackProperties *propertie
 	xmmsv_dict_iter_pair (parent_it, &key, &child_value);
 
 	/* check type of property */
-	if (strcmp( key, "artist" ) == 0)
-	    val_str = &(properties->artist);
-	else if (strcmp( key, "album" ) == 0)
-	    val_str = &(properties->album);
-	else if (strcmp( key, "mime" ) == 0)
-	    val_str = &(properties->mime);
-	else if (strcmp( key, "comment" ) == 0)
-	    val_str = &(properties->comment);
-	else if (strcmp( key, "channel" ) == 0)
-	    val_str = &channel;	    	    
-	else if (strcmp( key, "url" ) == 0)
-	    val_str = &(properties->url);	    
-	else if (strcmp( key, "title" ) == 0) 
-	    val_str = &(properties->title);	    
-	else if (strcmp( key, "duration" ) == 0)
-	    val_int = &(properties->duration);
-	else if (strcmp( key, "isvbr" ) == 0)
-	    val_int = &(properties->isvbr);
-	else if (strcmp( key, "bitrate" ) == 0)
-	    val_int = &(properties->bitrate);
-	else if (strcmp( key, "size" ) == 0)
-	    val_int = &(properties->size);
-	
+	MAYBE_GET_PROPERTY(properties, artist, key, val_str);
+	MAYBE_GET_PROPERTY(properties, album, key, val_str);
+	MAYBE_GET_PROPERTY(properties, mime, key, val_str);
+	MAYBE_GET_PROPERTY(properties, comment, key, val_str);
+	MAYBE_GET_PROPERTY(properties, url, key, val_str);
+	MAYBE_GET_PROPERTY(properties, title, key, val_str);
+	MAYBE_GET_PROPERTY(properties, picture_front, key, val_str);
+	MAYBE_GET_PROPERTY(properties, duration, key, val_int);
+	MAYBE_GET_PROPERTY(properties, isvbr, key, val_int);
+	MAYBE_GET_PROPERTY(properties, bitrate, key, val_int);
+	MAYBE_GET_PROPERTY(properties, size, key, val_int);
+
 	/* check if we got a dict_of_dict */
 	if (xmmsv_is_type (child_value, XMMSV_TYPE_DICT) )
 	{
@@ -1320,8 +1457,11 @@ static gboolean get_track_properties (xmmsv_t *value, TrackProperties *propertie
 static void queue_update_track( uint32_t id, GtkTreeIter* it )
 {
     xmmsc_result_t *res = xmmsc_medialib_get_info( con, id );
-    xmmsc_result_notifier_set_full ( res, (xmmsc_result_notifier_t)update_track, it, (
-					 xmmsc_user_data_free_func_t) gtk_tree_iter_free );
+    GtkTreePath *path = gtk_tree_model_get_path(GTK_TREE_MODEL(list_store), it);
+    GtkTreeRowReference *ref = gtk_tree_row_reference_new(GTK_TREE_MODEL(list_store), path);
+    gtk_tree_path_free(path);
+    xmmsc_result_notifier_set_full ( res, (xmmsc_result_notifier_t)update_track, ref, (
+					 xmmsc_user_data_free_func_t) gtk_tree_row_reference_free );
     xmmsc_result_unref( res );
 }
 
@@ -1329,8 +1469,8 @@ static int on_playlist_entries_received( xmmsv_t* value, GtkWidget* list_view )
 {
     GtkTreeModel* mf;
     GtkTreeIter it;
-    xmmsv_coll_t *coll;
     xmmsc_result_t *res;
+
     int i;
     int pl_size = xmmsv_list_get_size( value);;
 
@@ -1356,8 +1496,8 @@ static int on_playlist_entries_received( xmmsv_t* value, GtkWidget* list_view )
     res = xmmsc_coll_get( con, cur_playlist, "Playlists" );
     xmmsc_result_notifier_set_and_unref(res, on_playlist_coll_received, NULL );
 								
-    if( GTK_WIDGET_REALIZED( list_view ) )
-        gdk_window_set_cursor( list_view->window, NULL );
+    if( gtk_widget_get_realized( list_view ) )
+        gdk_window_set_cursor( gtk_widget_get_window(list_view), NULL );
 
     gtk_tree_view_set_model( GTK_TREE_VIEW(list_view), mf );
 
@@ -1386,10 +1526,10 @@ static void update_play_list( GtkWidget* list_view )
 {
     xmmsc_result_t *res;
 
-    if( GTK_WIDGET_REALIZED( list_view ) ) {
+    if( gtk_widget_get_realized( list_view ) ) {
         GdkCursor* cur;
         cur = gdk_cursor_new( GDK_WATCH );
-        gdk_window_set_cursor( list_view->window, cur );
+        gdk_window_set_cursor( gtk_widget_get_window(list_view), cur );
         gdk_cursor_unref( cur );
     }
     /* get current playlist as id_list */
@@ -1620,12 +1760,12 @@ static int on_playlist_content_changed( xmmsv_t* value, void* user_data )
     if( xmmsv_is_error( value ) )
 	return TRUE;
 
-    xmmsv_dict_get( value, "type", &int_value );
-    if( !xmmsv_get_int( int_value, &type ) )
+    if (!xmmsv_dict_get( value, "type", &int_value ) ||
+        !xmmsv_get_int( int_value, &type ))
         return TRUE;
     
-    xmmsv_dict_get( value, "name", &string_value );
-    if ( !xmmsv_get_string( string_value, &name ) )
+    if (!xmmsv_dict_get( value, "name", &string_value ) ||
+        !xmmsv_get_string( string_value, &name ))
 	return TRUE;
 
     if( ! name || !cur_playlist || strcmp(name, cur_playlist) )
@@ -1651,7 +1791,7 @@ static int on_playlist_content_changed( xmmsv_t* value, void* user_data )
 		xmmsv_get_int( int_value, &id );
                 gtk_list_store_insert_with_values( list_store, &it, pos, COL_ID, id, -1 );
                 /* g_debug("playlist_added: %d", id); */
-                queue_update_track( id, gtk_tree_iter_copy( &it ));
+                queue_update_track( id, &it );
             }
             break;
         case XMMS_PLAYLIST_CHANGED_REMOVE:
@@ -1664,10 +1804,14 @@ static int on_playlist_content_changed( xmmsv_t* value, void* user_data )
 		{
                     gtk_list_store_remove( list_store, &it );
 		    /* invalidate currently played track */
-		    xmmsv_dict_get( value, "id", &int_value );
-		    xmmsv_get_int( int_value, &id );
-		    if ( id == cur_track_id ) 
-			cur_track_id = 0;
+		    if (xmmsv_dict_get( value, "id", &int_value ))
+		    {
+		        xmmsv_get_int( int_value, &id );
+		        if ( id == cur_track_id ) 
+			    cur_track_id = 0;
+		    }
+		    else
+		        cur_track_id = 0;
 		}
                 gtk_tree_path_free( path );
 	    }
@@ -1787,37 +1931,39 @@ static int on_playback_track_loaded( xmmsv_t* value, void* user_data )
     gtk_window_set_title( GTK_WINDOW(main_win), window_title->str );
     
     if( tray_icon )
-        gtk_status_icon_set_tooltip(GTK_STATUS_ICON(tray_icon), window_title->str);
-
-    send_notifcation( track_properties.artist, track_properties.title );
+        gtk_status_icon_set_tooltip_text(GTK_STATUS_ICON(tray_icon), window_title->str);
 
+    LXMusicNotification lxn = lxmusic_do_notify_prepare ( track_properties.artist, track_properties.title,  
+							      _("Now Playing:"), GTK_STATUS_ICON(tray_icon) );
+    if (!track_properties.picture_front) 
+	send_notification( lxn );
+    else 
+    {
+	/* postpone notification: wait for album art */
+	xmmsc_result_t* res = xmmsc_bindata_retrieve( con, track_properties.picture_front );
+	xmmsc_result_notifier_set_and_unref (res, on_picture_front_received, lxn );
+    }
     g_string_free( window_title, TRUE );
     return TRUE;
 }
 
 
-static void send_notifcation( const gchar *artist, const gchar* title ) 
+static void send_notification_pixbuf( LXMusicNotification lxn, GdkPixbuf *pixbuf ) 
 {
-#ifdef HAVE_LIBNOTIFY
-    if( ! GTK_WIDGET_VISIBLE(main_win) ) 
+    if(!gtk_widget_get_visible(main_win))
     {
-	GString* notification_message = g_string_new("");
-	
-	if ( (artist != NULL) && (title != NULL ) ) {	
-	    /* metadata available */
-	    g_string_append_printf(notification_message, "<b>%s: </b><i>%s</i>", _("Artist"), artist );
-	    g_string_append_printf(notification_message, "\n<b>%s: </b><i>%s</i>", _("Title"), title );
-	}
-	/* use filename without markup */
-	else 			
-	    g_string_append( notification_message, title );
-	lxmusic_do_notify ( lxmusic_notification, _("Now Playing:"), notification_message->str );
-	g_string_free( notification_message, TRUE );
+	/* FIXME: Hardcoded notification icon size */
+	GdkPixbuf *scaled_pixbuf = gdk_pixbuf_scale_simple( pixbuf, 64, 64, GDK_INTERP_HYPER );
+	lxmusic_do_notify_pixbuf( lxn, scaled_pixbuf );
+	g_object_unref( scaled_pixbuf );
     }
-#endif	/* HAVE_LIBNOTIFY */
 }
 
-
+static void send_notification( LXMusicNotification lxn ) 
+{
+    if(!gtk_widget_get_visible(main_win))
+	lxmusic_do_notify( lxn );
+}
 
 
 static int on_playback_cur_track_changed( xmmsv_t* value, void* user_data )
@@ -1859,8 +2005,8 @@ static int on_playlist_pos_changed( xmmsv_t* val, void* user_data )
                     gtk_list_store_set(list_store, &it, COL_WEIGHT, PANGO_WEIGHT_BOLD, -1);
 
 		    /* update id of currently played track */
-		    xmmsv_dict_get( val, "id", &int_value );
-		    xmmsv_get_int( int_value, &cur_track_id );
+                    if( xmmsv_dict_get( val, "id", &int_value ) )
+                        xmmsv_get_int( int_value, &cur_track_id );
                 }
 		/* scroll to currently played song */
 		gtk_tree_view_scroll_to_cell(GTK_TREE_VIEW(playlist_view), path, NULL, FALSE, 0.0, 0.0 );
@@ -1918,7 +2064,7 @@ static void get_channel_volumes( const char *key, xmmsv_t *value, void *user_dat
     GSList** volumes = (GSList**)user_data;
     int32_t volume;
     xmmsv_get_int( value , &volume );
-    *volumes = g_slist_prepend(*volumes, (gpointer)volume);
+    *volumes = g_slist_prepend(*volumes, GINT_TO_POINTER(volume));
 }
 
 static void get_channel_volume_names(const char* key, xmmsv_t *value, void *user_data)
@@ -1954,7 +2100,7 @@ static void on_volume_btn_changed(GtkScaleButton* btn, gdouble val, gpointer use
 
 static void on_volume_btn_scrolled(GtkWidget *widget, GdkEventScroll *event, gpointer user_data)
 {
-	guint volume;
+	gshort volume;
 	GtkAdjustment *vol_adj;
 	xmmsc_result_t *res;
 	res = xmmsc_playback_volume_get(con);
@@ -1965,10 +2111,16 @@ static void on_volume_btn_scrolled(GtkWidget *widget, GdkEventScroll *event, gpo
 	{
 		case GDK_SCROLL_UP:
 			volume = gtk_adjustment_get_value (vol_adj) + 2;
+			if ( volume >= 100 ){
+			  volume = 100;
+			}
 			gtk_adjustment_set_value (GTK_ADJUSTMENT(vol_adj), volume);
 			break;
 		case GDK_SCROLL_DOWN:
 			volume = gtk_adjustment_get_value (vol_adj) - 2;
+			if ( volume <= 0 ){
+			  volume = 0;
+			}
 			gtk_adjustment_set_value (GTK_ADJUSTMENT(vol_adj), volume);
 			break;
 		default:
@@ -1988,7 +2140,7 @@ static int on_playback_volume_changed( xmmsv_t* value, void* user_data )
 	xmmsv_get_error( value, &error_msg );
 	g_warning( "%s: %s", __func__, error_msg );
     }
-    else if (xmmsv_is_type( value, XMMSV_TYPE_DICT ))
+    else if (value != NULL && xmmsv_is_type( value, XMMSV_TYPE_DICT ))
     {
 	xmmsv_dict_foreach( value, get_channel_volumes, &volumes );
         for( l = volumes; l; l = l->next )
@@ -2011,18 +2163,18 @@ static int on_collection_changed( xmmsv_t* dict, void* user_data )
     /* xmmsc_result_dict_foreach(res, dict_foreach, NULL); */
     
     xmmsv_t *string_value, *int_value;
-    const char *name, *ns;
-    gint32 type;
+    const char *name = NULL, *ns = NULL;
+    gint32 type = -1;
     
     
-    xmmsv_dict_get (dict, "name", &string_value);
-    xmmsv_get_string( string_value, &name);
+    if (xmmsv_dict_get (dict, "name", &string_value))
+        xmmsv_get_string( string_value, &name);
 
-    xmmsv_dict_get (dict, "namespace", &string_value);
-    xmmsv_get_string( string_value, &ns );
+    if (xmmsv_dict_get (dict, "namespace", &string_value))
+        xmmsv_get_string( string_value, &ns );
 
-    xmmsv_dict_get (dict, "type", &int_value);	
-    xmmsv_get_int( int_value, &type );
+    if (xmmsv_dict_get (dict, "type", &int_value))
+        xmmsv_get_int( int_value, &type );
     
     /* g_debug("name=%s, ns=%s, type=%d", name, ns, type); */
 
@@ -2068,7 +2220,7 @@ static int on_media_lib_entry_changed(xmmsv_t* value, void* user_data)
                 if( _id == id )
                 {
                     /* g_debug("found! update: %d", id); */
-                    queue_update_track( id, gtk_tree_iter_copy( &it ) );
+                    queue_update_track( id, &it );
                     break;
                 }
             }while(gtk_tree_model_iter_next(model, &it));
@@ -2244,7 +2396,7 @@ static void setup_ui()
     /* add volume button */
     hbox = (GtkWidget*)gtk_builder_get_object(builder, "top_hbox");
     volume_btn = gtk_volume_button_new();
-    gtk_scale_button_get_adjustment(GTK_SCALE_BUTTON(volume_btn))->upper = 100;
+    gtk_adjustment_set_upper (gtk_scale_button_get_adjustment(GTK_SCALE_BUTTON(volume_btn)), 100);
     gtk_widget_show(volume_btn);
     gtk_box_pack_start(GTK_BOX(hbox), volume_btn, FALSE, TRUE, 0);
     g_signal_connect(volume_btn, "value-changed", G_CALLBACK(on_volume_btn_changed), NULL);
@@ -2257,7 +2409,7 @@ static void setup_ui()
     gtk_builder_connect_signals(builder, NULL);
 
     /* window icon */
-    gtk_window_set_icon_from_file(GTK_WINDOW(main_win), PACKAGE_DATA_DIR"/pixmaps/lxmusic.png", NULL );
+    gtk_window_set_icon_name(GTK_WINDOW(main_win), "lxmusic");
 
     gtk_window_set_default_size(GTK_WINDOW(main_win), win_width, win_height);
     /* this can trigger signal handler and show or hide the playlist. */
@@ -2279,8 +2431,9 @@ void on_new_playlist(GtkAction* act, gpointer user_data)
             _("Create new playlist"), (GtkWindow*)main_win, GTK_DIALOG_MODAL,
             GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
             GTK_STOCK_OK, GTK_RESPONSE_OK, NULL );
+    gtk_window_set_destroy_with_parent(GTK_WINDOW(dlg), TRUE);
     GtkWidget *entry = gtk_entry_new();
-    gtk_box_pack_start( (GtkBox*)((GtkDialog*)dlg)->vbox, entry, FALSE, FALSE, 4 );
+    gtk_box_pack_start( (GtkBox*)gtk_dialog_get_content_area(GTK_DIALOG(dlg)), entry, FALSE, FALSE, 4 );
     gtk_dialog_set_default_response( (GtkDialog*)dlg, GTK_RESPONSE_OK );
     gtk_entry_set_activates_default( (GtkEntry*)entry, TRUE );
     gtk_widget_show_all( dlg );
@@ -2337,7 +2490,7 @@ void on_del_playlist(GtkAction* act, gpointer user_data)
 void on_show_playlist(GtkAction* act, gpointer user_data)
 {
     show_playlist = gtk_toggle_action_get_active(GTK_TOGGLE_ACTION(act));
-    if(GTK_WIDGET_VISIBLE(inner_vbox))
+    if(gtk_widget_get_visible(inner_vbox))
     {
         if( ! show_playlist )
         {
@@ -2408,12 +2561,6 @@ int main (int argc, char *argv[])
     /* build the GUI */
     setup_ui();
 
-#ifdef HAVE_LIBNOTIFY
-    if (!notify_is_initted ())
-	notify_init ("LXMusic");
-    lxmusic_notification  = lxmusic_notification_new( GTK_STATUS_ICON( tray_icon ) );
-#endif
-
     /* some dirty hacks to show the window earlier :-D */
     while( gtk_events_pending() )
         gtk_main_iteration();
